<?php

require_once(dirname(__FILE__).'/cache.php');
require_once(dirname(__FILE__).'/const.php');
require_once(dirname(__FILE__).'/migrator.php');

define('DiscloseSecret_ACTION_UPDATE_OPTIONS', 'update_options');
define('DiscloseSecret_CONFIG_FORM_ID', 'disclosesecret_configuration');

// ----------------------------------------------------------------------------
// the DiscloseSecret_ConfigUi class displays the config interface in the
// WP admin panel
// ----------------------------------------------------------------------------

// use this guard to avoid error messages in WP admin panel if plugin
// is disabled because of a version conflict but you still try to reload
// the plugins config interface
if (!class_exists('DiscloseSecret_ConfigUi')) :
class DiscloseSecret_ConfigUi
{
	var $m_cache;
	var $m_migrator;

	// ----------------------------------------------------------------------------
	// init
	// ----------------------------------------------------------------------------

	// Sets up the Disclose-Secret config menu
	function DiscloseSecret_ConfigUi(&$cache, $activation_status)
	{
		$this->m_cache = $cache;
		$this->toggle_filter($activation_status);
		add_filter('admin_head', array(&$this, 'filter_admin_head'));
	}

	function toggle_filter($activation_status)
	{
		if ($activation_status == DiscloseSecret_STATUS_OKAY
			|| $activation_status == DiscloseSecret_STATUS_WP_VERSION_MISMATCH)
			add_filter('admin_menu', array(&$this, 'filter_admin_menu_option'));
		else
			remove_filter('admin_menu', array(&$this, 'filter_admin_menu_option'));

		if ($activation_status == DiscloseSecret_STATUS_WP_VERSION_MISMATCH)
			add_filter('admin_footer', array(&$this, 'filter_admin_footer_wp_version'));
		else
			remove_filter('admin_footer', array(&$this, 'filter_admin_footer_wp_version'));

		if ($activation_status == DiscloseSecret_STATUS_PLUGIN_VERSION_MISMATCH)
			add_filter('admin_footer', array(&$this, 'filter_admin_footer_plugin_version'));
		else
			remove_filter('admin_footer', array(&$this, 'filter_admin_footer_plugin_version'));

		if ($activation_status == DiscloseSecret_STATUS_SQL_VERSION_MISMATCH)
			add_filter('admin_footer', array(&$this, 'filter_admin_footer_mysql_version'));
		else
			remove_filter('admin_footer', array(&$this, 'filter_admin_footer_mysql_version'));

		if ($activation_status != DiscloseSecret_STATUS_OKAY
			&& $activation_status != DiscloseSecret_STATUS_WP_VERSION_MISMATCH
			&& $activation_status != DiscloseSecret_STATUS_PLUGIN_VERSION_MISMATCH
			&& $activation_status != DiscloseSecret_STATUS_SQL_VERSION_MISMATCH)
			add_filter('admin_footer', array(&$this, 'filter_admin_footer_unknown'));
		else
			remove_filter('admin_footer', array(&$this, 'filter_admin_footer_unknown'));
	}

	function filter_admin_menu_option()
	{
		// delay initialization until the WP framework is loaded
		$this->m_migrator =& new DiscloseSecret_Migrator();
		add_submenu_page('options-general.php',
			DiscloseSecret_PLUGIN_NAME,
			DiscloseSecret_PLUGIN_NAME,
			'edit_others_posts', __FILE__,
			array(&$this, 'submenu_page_option_general'));
	}

	function filter_admin_head()
	{
?>
	<script type="text/javascript">
		function DiscloseSecret_setMessage(heading, text)
		{
			var message = "<p><strong>" + heading + "</strong> " + text + "</p>";
			var container = document.getElementById("disclose-secret-message");
			try
			{
				container.innerHTML = container.innerHTML + message;
			}
			catch(e)
			{
				container = document.createElement("div");
				container.className = "updated fade-ff0000";
				container.setAttribute("id", "disclose-secret-message");
				var adminmenu = document.getElementById("adminmenu");
				adminmenu.parentNode.insertBefore(container, adminmenu.nextSibling);
				container.innerHTML = message;
			}
		}
	</script>

<?php
	}

	function filter_admin_footer_wp_version()
	{
		$option =& $this->m_cache->get_option();

		$heading = sprintf(__('%s plugin is not active.', DiscloseSecret_PLUGIN_ID)
			, DiscloseSecret_PLUGIN_NAME);
		$text = sprintf(__('For security reasons the %s plugin functionality was turned off because it was only tested with the following WordPress versions: %s. All protected content becomes private. You can forcefully turn on this plugin on the plugins configuration page.', DiscloseSecret_PLUGIN_ID)
			, DiscloseSecret_PLUGIN_NAME, implode(', ', $option->get_allowed_wp_versions()));
		$this->print_admin_message($heading, $text);
	}

	function filter_admin_footer_plugin_version()
	{
		$option =& $this->m_cache->get_option();

		$heading = sprintf(__('%s plugin is not active.', DiscloseSecret_PLUGIN_ID)
			, DiscloseSecret_PLUGIN_NAME);
		$text = sprintf(__('For security reasons the %s plugin functionality was turned off because no necessary upgrade of the plugin could be performed. All protected content becomes private. This is plugin version %s but previously there was version %s installed. Downgrading from a newer version to an older version of the plugin is not supported.', DiscloseSecret_PLUGIN_ID)
			, DiscloseSecret_PLUGIN_NAME, DiscloseSecret_VERSION, $option->get_version());
		$this->print_admin_message($heading, $text);
	}

	function filter_admin_footer_mysql_version()
	{
		$option =& $this->m_cache->get_option();

		$heading = sprintf(__('%s plugin is not active.', DiscloseSecret_PLUGIN_ID)
			, DiscloseSecret_PLUGIN_NAME);
		$text = sprintf(__('For security reasons the %s plugin functionality was turned off because you need at least MySQL server version 4.1 to be installed but only %s is available. All protected content becomes private.', DiscloseSecret_PLUGIN_ID)
			, DiscloseSecret_PLUGIN_NAME, mysql_get_server_info());
		$this->print_admin_message($heading, $text);
	}

	function filter_admin_footer_unknown()
	{
		$option =& $this->m_cache->get_option();
		$heading = sprintf(__('%s plugin is not active.', DiscloseSecret_PLUGIN_ID)
			, DiscloseSecret_PLUGIN_NAME);
		$text = sprintf(__('For security reasons the %s plugin functionality was turned off because an unknown error (%s) occured. All protected content becomes private. This should never happen if you use the plugin with a compatible WordPress version and installed it as described in the documentation.', DiscloseSecret_PLUGIN_ID)
			, DiscloseSecret_PLUGIN_NAME, $option->get_cooperation_status());
		$this->print_admin_message($heading, $text);
	}

	// ----------------------------------------------------------------------------
	// tools
	// ----------------------------------------------------------------------------

	function print_admin_message($heading, $text)
	{
?>
	<script type="text/javascript">
		DiscloseSecret_setMessage('<?php echo $heading; ?>', '<?php echo $text; ?>');
	</script>

<?php
	}

	function print_feed_privacy($feed_options, $feed_privacy)
	{
		foreach ($feed_options as $key => $value)
		{
			$selected = "";
			if ($key == $feed_privacy)
				$selected = " selected='yes'";
?>
			<option value="<?php echo $key; ?>"<?php echo $selected; ?>><?php echo $value; ?></option>
<?php
		}
	}

	// ----------------------------------------------------------------------------
	// interface
	// ----------------------------------------------------------------------------

	// Disclose-Secret configuration page
	function submenu_page_option_general()
	{
		global $wpdb;

		// executing form actions
		$option =& $this->m_cache->get_option();
		if (isset($_POST[DiscloseSecret_ACTION_UPDATE_OPTIONS]))
		{
			check_admin_referer(DiscloseSecret_ACTION_UPDATE_OPTIONS);
			$option->set_from_POST();
			$option->save();
			echo '<div id="message" class="updated fade"><p><strong>'.
				__('Options saved.', DiscloseSecret_PLUGIN_ID) . "</strong></p></div>\n";
		}
		$this->toggle_filter($option->get_cooperation_status());

		if ($this->m_migrator->migrate())
		{
			echo '<div id="message" class="updated fade"><p><strong>'.
				__('Plugin migrated', DiscloseSecret_PLUGIN_ID) . "</strong></p></div>\n";
		}
?>
	<div class="wrap">
		<h2><?php echo sprintf(__('Disclose-Secret %s Options', DiscloseSecret_PLUGIN_ID), DiscloseSecret_VERSION); ?></h2>
		<p><?php echo sprintf(__("Disclose-Secret shows articles to other users if the user meets certain access criteria. Access to articles can be individually restricted on the 'Write' dialog on a per user, role and/or capability base. A <a href='%s'>local copy of the documentation</a> comes with this plugin. New versions and further documentation may be found on the <a href='http://bluesome.net/post/2006/01/01/121/'>official plugin page</a>.", DiscloseSecret_PLUGIN_ID), get_option('siteurl'). '/'. DiscloseSecret_DIR. '/doc/readme.html'); ?></p>
		<form action="" method="post" id="<?php echo DiscloseSecret_CONFIG_FORM_ID; ?>">
<?php if (!$option->is_wp_version_compatible()) : ?>
			<fieldset class="options">
				<legend><?php _e('Basic options', DiscloseSecret_PLUGIN_ID); ?></legend>
				<p><?php _e("The basic options define the overall behavior of the plugin.", DiscloseSecret_PLUGIN_ID); ?></p>
				<table class="editform optiontable">
					<tr valign="top">
						<th scope="row"><?php _e('Forcefully turn on:', DiscloseSecret_PLUGIN_ID); ?></th>
						<td>
							<label for="<?php echo DiscloseSecret_POST_OVERRIDE_PLUGIN_DEACTIVATION; ?>">
								<input type="checkbox" name="<?php echo DiscloseSecret_POST_OVERRIDE_PLUGIN_DEACTIVATION; ?>" id="<?php echo DiscloseSecret_POST_OVERRIDE_PLUGIN_DEACTIVATION; ?>" value="true" <?php if ($option->get_override_plugin_deactivation()) : ?>checked="checked" <?php endif; ?>/>
								<?php echo sprintf(__('In case you are using the plugin with an unsupported version of WordPress the plugin turns off for security reasons. You can forcefully turn on the plugin by setting this option. Remember that this option will be deselected each time you either update WordPress or this plugin.', DiscloseSecret_PLUGIN_ID)); ?>

							</label>
						</td>
					</tr>
				</table>
			</fieldset>
<?php endif; ?>

			<fieldset class="options">
				<legend><?php _e('Write options', DiscloseSecret_PLUGIN_ID); ?></legend>
				<p><?php _e("With the Write options you can specify how your 'Write' dialog in the admin interface should appear.", DiscloseSecret_PLUGIN_ID); ?></p>
				<table class="editform optiontable">
					<tr valign="top">
						<th scope="row"><?php _e('Show &quot;Access by user&quot; box:', DiscloseSecret_PLUGIN_ID); ?></th>
						<td>
							<label for="<?php echo DiscloseSecret_POST_PRINT_USER_FORM; ?>">
								<input type="checkbox" name="<?php echo DiscloseSecret_POST_PRINT_USER_FORM; ?>" id="<?php echo DiscloseSecret_POST_PRINT_USER_FORM; ?>" value="true" <?php if ($option->get_print_user_form()) : ?>checked="checked" <?php endif; ?>/>
								<?php echo sprintf(__("In case you experience performance problems when displaying your 'Write' dialog, you may want to turn off displaying this box. This does not disable setting permissions based on users. It only turns off the selection box.", DiscloseSecret_PLUGIN_ID)); ?>

							</label>
						</td>
					</tr>

					<tr valign="top">
						<th scope="row"><?php _e('Show &quot;Access by role&quot; box:', DiscloseSecret_PLUGIN_ID); ?></th>
						<td>
							<label for="<?php echo DiscloseSecret_POST_PRINT_ROLE_FORM; ?>">
								<input type="checkbox" name="<?php echo DiscloseSecret_POST_PRINT_ROLE_FORM; ?>" id="<?php echo DiscloseSecret_POST_PRINT_ROLE_FORM; ?>" value="true" <?php if ($option->get_print_role_form()) : ?>checked="checked" <?php endif; ?>/>
								<?php echo sprintf(__("In case you experience performance problems when displaying your 'Write' dialog, you may want to turn off displaying this box. This does not disable setting permissions based on roles. It only turns off the selection box.", DiscloseSecret_PLUGIN_ID)); ?>

							</label>
						</td>
					</tr>

					<tr valign="top">
						<th scope="row"><?php _e('Show &quot;Access by capability&quot; box:', DiscloseSecret_PLUGIN_ID); ?></th>
						<td>
							<label for="<?php echo DiscloseSecret_POST_PRINT_CAP_FORM; ?>">
								<input type="checkbox" name="<?php echo DiscloseSecret_POST_PRINT_CAP_FORM; ?>" id="<?php echo DiscloseSecret_POST_PRINT_CAP_FORM; ?>" value="true" <?php if ($option->get_print_cap_form()) : ?>checked="checked" <?php endif; ?>/>
								<?php echo sprintf(__("In case you experience performance problems when displaying your 'Write' dialog, you may want to turn off displaying this box. This does not disable setting permissions based on capabilities. It only turns off the selection box.", DiscloseSecret_PLUGIN_ID)); ?>

							</label>
						</td>
					</tr>
				</table>
			</fieldset>

			<fieldset class="options">
				<legend><?php _e('Newsfeed options', DiscloseSecret_PLUGIN_ID); ?></legend>
				<p><?php _e("With the newsfeed options you can configure the behaviour of your newsfeeds when they are read by logged in users. If the user is not logged in, neither protected nor private articles will appear in your newsfeed.", DiscloseSecret_PLUGIN_ID); ?></p>
				<table class="editform optiontable">
					<tr valign="top">
						<th scope="row"><?php _e('Default Feed Privacy:', DiscloseSecret_PLUGIN_ID); ?></th>
						<td>
							<select name="<?php echo DiscloseSecret_POST_FEED_PRIVACY; ?>" id="<?php echo DiscloseSecret_POST_FEED_PRIVACY; ?>">
<?php $this->print_feed_privacy($option->get_feed_options(), $option->get_feed_privacy()); ?>
							</select>
							<br />
							<?php _e("For logged in users reading feeds via HTTP authentication, this decides how much content of a protected or private article should be shown in the newsfeed. Because newsfeeds may get cached by newsreader programs, it may be preferable to hide some or all potions of an article in the newsfeeds to avoid exposing content.", DiscloseSecret_PLUGIN_ID); ?>

						</td>
					</tr>

					<tr valign="top">
						<th scope="row"><?php _e('Feed Teaser:', DiscloseSecret_PLUGIN_ID); ?></th>
						<td>
							<textarea name="<?php echo DiscloseSecret_POST_FEED_TEASER; ?>" id="<?php echo DiscloseSecret_POST_FEED_TEASER; ?>" cols="60" rows="4" style="width: 98%; font-size: 12px;" class="code"><?php echo htmlspecialchars($option->get_feed_teaser(), ENT_QUOTES); ?></textarea>
							<?php _e('Define the teaser text that is shown to logged in users accessing your newsfeed in case not the full content or the excerpt is shown. This text will show up for each protected and private article. You can use the following placeholders for certain dynamic text:', DiscloseSecret_PLUGIN_ID); ?>

							<ul>
								<li><?php echo DiscloseSecret_PLACEHOLDER_PERMALINK; ?> - causing the permalink URL to be inserted</li>
							</ul>
						</td>
					</tr>

					<tr valign="top">
						<th scope="row"><?php _e('Rewrite Feed Links:', DiscloseSecret_PLUGIN_ID); ?></th>
						<td>
							<label for="<?php echo DiscloseSecret_POST_REWRITE_FEED_LINK; ?>">
								<input type="checkbox" name="<?php echo DiscloseSecret_POST_REWRITE_FEED_LINK; ?>" id="<?php echo DiscloseSecret_POST_REWRITE_FEED_LINK; ?>" value="true" <?php if ($option->get_rewrite_feed_link()) : ?>checked="checked" <?php endif; ?>/>
								<?php echo sprintf(__('For logged in users, automatically appends <code>%s=%s</code> to the query string of feed links to support HTTP Authentication. This for convenience to your readers.', DiscloseSecret_PLUGIN_ID), DiscloseSecret_POST_HTTP_AUTH, DiscloseSecret_POST_HTTP_AUTH_DO); ?>

							</label>
						</td>
					</tr>

				</table>
			</fieldset>

			<p class="submit">
				<input type="submit" name="<?php echo DiscloseSecret_ACTION_UPDATE_OPTIONS; ?>" value="<?php _e('Update Options') ?> &raquo;" />
				<?php wp_nonce_field(DiscloseSecret_ACTION_UPDATE_OPTIONS); ?>

			</p>
		</form>
	</div>
<?php $this->m_migrator->print_form() ?>

<?php
	}
}
endif;

?>