<?php

require_once(dirname(__FILE__).'/cache.php');
require_once(dirname(__FILE__).'/const.php');
require_once(dirname(__FILE__).'/pcre.php');
require_once(dirname(__FILE__).'/php.php');

define('DiscloseSecret_XHTML_TAG', 'disclose-secret');
define('DiscloseSecret_XHTML_TAG_ALT_1', 'disclosesecret');
define('DiscloseSecret_XHTML_TAG_ALT_2', 'disclose_secret');
define('DiscloseSecret_XHTML_ATTRIBUTE_CAP', 'cap');
define('DiscloseSecret_XHTML_ATTRIBUTE_CAP_ALT_1', 'caps');
define('DiscloseSecret_XHTML_ATTRIBUTE_ROLE', 'role');
define('DiscloseSecret_XHTML_ATTRIBUTE_ROLE_ALT_1', 'roles');
define('DiscloseSecret_XHTML_ATTRIBUTE_USER', 'user');
define('DiscloseSecret_XHTML_ATTRIBUTE_USER_ALT_1', 'users');

// ----------------------------------------------------------------------------
// the DiscloseSecret_Runtime class handles the checks for accessing a
// restricted article, saving restrictions for an article and modifying the
// RSS feed
// ----------------------------------------------------------------------------

class DiscloseSecret_Runtime
{
	var $m_cache;
	var $m_save_as_private = false;
	var $m_filter_query_priority = 50;
	var $m_pcre;

	// ----------------------------------------------------------------------------
	// init
	// ----------------------------------------------------------------------------

	function DiscloseSecret_Runtime(&$cache, $activation_status)
	{
		$this->m_cache = $cache;
		$this->m_pcre = new DiscloseSecret_PCRE();

		if ($activation_status != DiscloseSecret_STATUS_OKAY)
			return;

		add_filter('content_save_pre', array(&$this, 'filter_content_save_pre'));
		add_filter('status_save_pre', array(&$this, 'filter_status_save_pre'));
		add_filter('save_post', array(&$this, 'filter_post_save'));

		add_filter('query', array(&$this, 'filter_query'), $this->m_filter_query_priority);
		add_filter('edit_category', array(&$this, 'filter_edit_category'));
		add_filter('user_has_cap', array(&$this, 'filter_user_has_cap'), 10, 3);

		$option =& $this->m_cache->get_option();
		if ($option->get_rewrite_feed_link())
			add_filter('feed_link', array(&$this, 'filter_feed_link'));
		if (isset($_GET[DiscloseSecret_POST_HTTP_AUTH])
			&& $_GET[DiscloseSecret_POST_HTTP_AUTH] == DiscloseSecret_POST_HTTP_AUTH_DO
			&& $this->m_feed_privacy != DiscloseSecret_OPTION_FULL_CONTENT)
		{
			// Only filter RSS content if we're using HTTP auth
			add_filter('the_content', array(&$this, 'filter_the_content_rss'));
			add_filter('the_content_rss', array(&$this, 'filter_the_content_rss'));
			add_filter('the_excerpt_rss', array(&$this, 'filter_the_excerpt_rss'));
		}
	}

	// ----------------------------------------------------------------------------
	// save article filter
	// ----------------------------------------------------------------------------

	function clear_tag_attribute($reference, $post_key)
	{
		foreach ($reference as $id => $name)
			unset($_POST[$post_key. $id]);
	}

	function marshal_tag_attribute($reference, $post_key, $list)
	{
		$a = explode(',', $list);
		foreach ($a as $item)
		{
			$item = trim($item);
			foreach ($reference as $id => $ref)
			{
				if (!strcasecmp($item, $ref[DiscloseSecret_INDEX_ID])
					|| !strcasecmp($item, $ref[DiscloseSecret_INDEX_NAME]))
				{
					$_POST[$post_key. $id] = 'on';
				}
			}
		}
	}

	function tag_to_POST($match)
	{
		global $post;

		$name_index = 1;
		$value_index = 3;
		$pattern = "/\s*(.*?)\s*=\s*([\'\"])((?:[^\\2\\\]|\\\.)*?)\\2/is";
		preg_match_all($pattern, stripslashes(trim($match[3])), $elements);

		$this->m_save_as_private = true;
		$count = count($elements[0]);

		// resetting values before parsing
		$option =& $this->m_cache->get_option();
		$this->clear_tag_attribute($option->get_caps(), DiscloseSecret_POST_CAP);
		$this->clear_tag_attribute($option->get_roles(), DiscloseSecret_POST_ROLE);
		$this->clear_tag_attribute($option->get_users(), DiscloseSecret_POST_USER);

		for ($n = 0; $n < $count; ++$n)
		{
			if (strtolower($elements[$name_index][$n]) == DiscloseSecret_XHTML_ATTRIBUTE_CAP
//				|| strtolower($elements[$name_index][$n]) == DiscloseSecret_XHTML_ATTRIBUTE_CAP_ALT_1
			)
			{
				$this->marshal_tag_attribute($option->get_caps(),
					DiscloseSecret_POST_CAP, $elements[$value_index][$n]);
			}
			else if (strtolower($elements[$name_index][$n]) == DiscloseSecret_XHTML_ATTRIBUTE_ROLE
//				|| strtolower($elements[$name_index][$n]) == DiscloseSecret_XHTML_ATTRIBUTE_ROLE_ALT_1
			)
			{
				$this->marshal_tag_attribute($option->get_roles(),
					DiscloseSecret_POST_ROLE, $elements[$value_index][$n]);
			}
			else if (strtolower($elements[$name_index][$n]) == DiscloseSecret_XHTML_ATTRIBUTE_USER
//				|| strtolower($elements[$name_index][$n]) == DiscloseSecret_XHTML_ATTRIBUTE_USER_ALT_1
			)
			{
				$this->marshal_tag_attribute($option->get_users(),
					DiscloseSecret_POST_USER, $elements[$value_index][$n]);
			}
		}
		return "";
	}

	// Called when an article is saved, evaluates article tags
	function filter_content_save_pre($content)
	{
		// extract possible in-article restriction settings to $_POST array
		// for later processing; last occurance of multiple tags
		// will overwrite previous tags
		$this->m_save_as_private = false;
		$pattern = "/<!--\s*(". DiscloseSecret_XHTML_TAG.
//			"|". DiscloseSecret_XHTML_TAG_ALT_1.
//			"|". DiscloseSecret_XHTML_TAG_ALT_2.
			")(\s+(.*?))?\s*\/?\s*-->/is";
		$content = preg_replace_callback($pattern, array(&$this, 'tag_to_POST'), $content);
		return $content;
	}

	// Called before article status is updated on save. Marks
	// article as private if a restriction is set
	function filter_status_save_pre($post_status)
	{
		if ($post_status == 'publish')
		{
			// this is for in-article tags
			if ($this->m_disclosesecret_tag_error)
				return 'private';

			// this is for admin panel settings
			$option =& $this->m_cache->get_option();
			foreach ($option->get_caps() as $cap)
			{
				if ($_POST[DiscloseSecret_POST_CAP. $cap[DiscloseSecret_INDEX_ID]])
					return 'private';
			}
			foreach ($option->get_roles() as $role)
			{
				if ($_POST[DiscloseSecret_POST_ROLE. $role[DiscloseSecret_INDEX_ID]])
					return 'private';
			}
			foreach ($option->get_users() as $user)
			{
				if ($_POST[DiscloseSecret_POST_USER. $user[DiscloseSecret_INDEX_ID]])
					return 'private';
			}
		}
		return $post_status;
	}

	// Called when an article is saved, updates the restriction
	function filter_post_save($post_ID)
	{
		$postmeta =& $this->m_cache->get_postmeta($post_ID);
		$postmeta->set_from_POST();
		$postmeta->save();
	}

	// ----------------------------------------------------------------------------
	// runtime filter
	// ----------------------------------------------------------------------------

	// Enable users with the right restriction to read a private article
	// This is required because single article view has special logic
	// $allcaps = Capabilities the user currently has
	// $caps = Primitive capabilities being tested / requested
	// $args = array with:
	// $args[0] = original meta capability requested
	// $args[1] = user being tested
	// $args[2] = post id to view
	// See code for assumptions

	// for pages this will work once patch #3705 from
	// trac.wordpress.org is incorporated into the WP release
	function filter_user_has_cap($allcaps, $caps, $args)
	{
		// This handler is only set up to deal with the read_private_posts
		// or read_private_pages capability. Ignore all other calls into here.
		$pages_request = in_array('read_private_pages', $caps);
		$posts_request = in_array('read_private_posts', $caps);
		if ((!$pages_request && !$posts_request)
			|| ($pages_request && $posts_request)
			|| !$args[0] || !$args[1] || !$args[2] || $args[1] == 0)
			return $allcaps;

		// As of WP 2.0, read_private_posts/pages is only requested when viewing
		// a single article. When this happens, the args[] array has three values,
		// as shown above. If WP changes the args[], this plugin will break

		$postmeta =& $this->m_cache->get_postmeta($args[2]);
		$intersect_cap = array_intersect(array_keys($allcaps), $postmeta->get_caps());
		$has_cap = !empty($intersect_cap);

		$current_user = wp_get_current_user();
		$has_role = false;
		$has_user = false;
		if (!empty($current_user))
		{
			$intersect_role = array_intersect($current_user->roles, $postmeta->get_roles());
			$has_role = !empty($intersect_role);
			$has_user = in_array($current_user->id, $postmeta->get_users());
		}

		$rpp_caps = array();
		$rpp_caps[$pages_request ? 'read_private_pages' : 'read_private_posts']
			= ($has_cap || $has_role || $has_user);
		return $rpp_caps;
	}

	// ----------------------------------------------------------------------------
	// column query manipulation
	// ----------------------------------------------------------------------------

	function replace_comment_approved($condition, $table_prefix, $quotation_value)
	{
		global $wpdb;
		$subquery =
			"({$condition} AND EXISTS (".
				"SELECT * FROM $wpdb->posts ".
				"WHERE {$table_prefix}comment_post_ID = $wpdb->posts.ID ".
					"AND ".
					$this->replace_equal_post_status("$wpdb->posts.post_status = 'publish'", "$wpdb->posts.", '\'').
			"))";
		return $subquery;
	}

	function replace_in_post_status($condition, $table_prefix, $quotation_value)
	{
		$pattern = "/([\'\"])((?:[^\\1\\\]|\\\.)*?)\\1/is";
		$subquery = preg_replace($pattern, "{$table_prefix}post_status = \\1\\2\\1", $quotation_value);
		$subquery = str_replace(array(' , ', ' ,', ', ', ','), " OR ", $subquery);
		$subquery = $this->m_pcre->parse(array(&$this, 'replace_in_query'), $subquery);
		return $subquery;
	}

	function replace_equal_post_status($condition, $table_prefix, $quotation_value)
	{
		global $current_user;
		$caps = implode("', '", array_keys($current_user->allcaps));
		if (!empty($caps))
			$caps = "meta_value IN ('$caps')";
		else
			$caps = '1=1';
		$roles = implode("', '", $current_user->roles);
		if (!empty($roles))
			$roles = "meta_value in ('$roles')";
		else
			$roles = '1=1';
		$user_id = $current_user->id;
		$users = "meta_value IN ('$user_id')";
		if (!is_numeric($user_id))
		{
			$user_id = 0;
			$users = '1=1';
		}

		global $wpdb;
		$subquery =
			"({$table_prefix}post_status = {$quotation_value}publish{$quotation_value} ".
				"OR ({$table_prefix}post_status = {$quotation_value}private{$quotation_value} ".
					"AND (EXISTS (SELECT * FROM $wpdb->postmeta WHERE meta_key = '". DiscloseSecret_META_CAP. "' ".
							"AND {$table_prefix}ID = $wpdb->postmeta.post_id AND ($caps)) ".
						"OR EXISTS (SELECT * FROM $wpdb->postmeta WHERE meta_key = '".DiscloseSecret_META_ROLE. "' ".
							"AND {$table_prefix}ID = $wpdb->postmeta.post_id AND ($roles)) ".
						"OR EXISTS (SELECT * FROM $wpdb->postmeta WHERE meta_key = '".DiscloseSecret_META_USER. "' ".
							"AND {$table_prefix}ID = $wpdb->postmeta.post_id AND ($users)) ".
						"OR {$table_prefix}post_author = {$user_id}".
					")".
				")".
			")";
		return $subquery;
	}

	// ----------------------------------------------------------------------------
	// generic query manipulation
	// ----------------------------------------------------------------------------

	function replace_in_query($match)
	{
		$string_marker = '';
		if (isset($match[DiscloseSecret_MATCH_VALUE_QUOTE][0]))
			$string_marker = $match[DiscloseSecret_MATCH_VALUE_QUOTE][0];

		$condition = '';
		if (isset($match[DiscloseSecret_MATCH_CONDITION][0]))
			$condition = $match[DiscloseSecret_MATCH_CONDITION][0];

		$table_prefix = '';
		if (isset($match[DiscloseSecret_MATCH_TABLE_PREFIX][0]))
			$table_prefix = $match[DiscloseSecret_MATCH_TABLE_PREFIX][0];

		$column_name = '';
		if (isset($match[DiscloseSecret_MATCH_COLUMN_NAME][0]))
			$column_name = $match[DiscloseSecret_MATCH_COLUMN_NAME][0];

		$operator = '';
		$quotation_value = '';
		$compare_value = '';
		if (isset($match[DiscloseSecret_MATCH_EQUAL_OPERATOR][0]))
		{
			$operator = $match[DiscloseSecret_MATCH_EQUAL_OPERATOR][0];
			$quotation_value = $match[DiscloseSecret_MATCH_EQUAL_QUOTE][0];
			$compare_value = $match[DiscloseSecret_MATCH_EQUAL_STRING][0];
		}
		else if (isset($match[DiscloseSecret_MATCH_IN_OPERATOR][0]))
		{
			$operator = $match[DiscloseSecret_MATCH_IN_OPERATOR][0];
			$quotation_value = $match[DiscloseSecret_MATCH_IN_QUOTED_STRING][0];
			$compare_value = '';
		}

		// skip all strings; just picked them out to avoid further pattern matching inside of them
		if (!($string_marker && ($string_marker == '\'' || $string_marker == '\"')))
		{
			$query_replace_callback = NULL;
			if (!strcasecmp($column_name, 'comment_approved')
				&& (!strcasecmp($operator, 'in') || !strcasecmp($operator, '=')))
				$query_replace_callback = array(&$this, 'replace_comment_approved');
			else if (!strcasecmp($column_name, 'post_status') && !strcasecmp($operator, '=')
				&& !strcasecmp($compare_value, 'publish'))
				$query_replace_callback = array(&$this, 'replace_equal_post_status');
			else if (!strcasecmp($column_name, 'post_status') && !strcasecmp($operator, 'in'))
				$query_replace_callback = array(&$this, 'replace_in_post_status');

			if ($query_replace_callback)
				$query = call_user_func_array($query_replace_callback,
					array($condition, $table_prefix, $quotation_value));
			else
				$query = $match[DiscloseSecret_MATCH_ALL][0];
		}
		else
			$query = $match[DiscloseSecret_MATCH_ALL][0];
		return $query;
	}

	function filter_query($query)
	{
		$query = $this->m_pcre->parse(array(&$this, 'replace_in_query'), $query);
		return $query;
	}

	function filter_edit_category($cat_id)
	{
		// avoid counting protected content for counts in wp_categories
		// SQL statements taken from post.php wp_set_post_categories()
		global $wpdb;
		remove_filter('query', array(&$this, 'filter_query'), $this->m_filter_query_priority);
		remove_filter('edit_category', array(&$this, 'filter_edit_category'));
		$count = $wpdb->get_var("SELECT COUNT(*) FROM $wpdb->post2cat, $wpdb->posts WHERE $wpdb->posts.ID=$wpdb->post2cat.post_id AND post_status = 'publish' AND post_type = 'post' AND category_id = '$cat_id'");
		$wpdb->query("UPDATE $wpdb->categories SET category_count = '$count' WHERE cat_ID = '$cat_id'");
		clean_category_cache($cat_id);
		do_action('edit_category', $cat_id);
		add_filter('edit_category', array(&$this, 'filter_edit_category'));
		add_filter('query', array(&$this, 'filter_query'), $this->m_filter_query_priority);
		return $cat_id;
	}

	// ----------------------------------------------------------------------------
	// rss filter
	// ----------------------------------------------------------------------------

	function replace_feed_teaser_placeholder($content)
	{
		global $post;

		$search[] = DiscloseSecret_PLACEHOLDER_PERMALINK;
		$replace[] = get_permalink($post->ID);
		$content = str_replace($search, $replace, $content);
		return $content;
	}

	// Called when using HTTP auth -- changes the article content
	function filter_the_content_rss($content)
	{
		global $post;

		if ($post->post_status != 'private')
			return $content;

		$option =& $this->m_cache->get_option();
		switch ($option->get_feed_privacy())
		{
			case DiscloseSecret_OPTION_EXCERPT_ONLY:
				return apply_filters('the_excerpt_rss', get_the_excerpt(true));
		}
		return $this->replace_feed_teaser_placeholder($option->get_feed_teaser());
	}

	// Called when using HTTP auth -- changes the article excerpt
	function filter_the_excerpt_rss($excerpt)
	{
		global $post;

		if ($post->post_status != 'private')
			return $excerpt;

		$option =& $this->m_cache->get_option();
		switch ($option->get_feed_privacy())
		{
			case DiscloseSecret_OPTION_EXCERPT_ONLY:
				return $excerpt;
		}
		return $this->replace_feed_teaser_placeholder($option->get_feed_teaser());
	}

	// Rewrites RSS feed links to support http authentication
	// if the user is logged in
	function filter_feed_link($output)
	{
		$delim = (strpos($output, '?') === false) ? '?' : '&';
		return $output. $delim. DiscloseSecret_POST_HTTP_AUTH. '='. DiscloseSecret_POST_HTTP_AUTH_DO;
	}
}

?>